#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "database.h"
#include "proxy.h"
#include "config.h"
#include "request.h"
#include "response.h"

#define PACKAGE_NAME "WICTP"
#define DEFAULT_DATABASE "data.sqlite"
#define DEFAULT_CONFIG "config.json"
#define DEFAULT_PORT 8080

unsigned int validatePortNumber(unsigned int portNumber){
	// Given that we are dealing with unsined ints, we don't need to worry about
	// checking for values less than 0
	if ( portNumber > 65535 ){
		printf( "Port %i is invalid, using default of %i", portNumber, DEFAULT_PORT );
		return DEFAULT_PORT;
	}
	return portNumber;
}

void printHelp(){
	printf("Usage: %s [options]\n", PACKAGE_NAME);
	printf( "Options are:\n" );
	printf( "\t --config OPTION VALUE      Sets config option OPTION to VALUE\n" );
	printf( "\t --print-config             Prints current config\n" );
}

int main(int argc, char**argv){
	Config *config = configDefaults();
	int listener;


	for ( unsigned int i = 1; i < argc; i++ ){
		if ( strcmp( argv[i], "--config" ) == 0 ){
			if ( i + 2 < argc ){
				setConfig( config, argv[i + 1], argv[i + 2] );
				i += 2;
			} else {
				printf("--config requires 2 positional arguments\n");
			}
		} else if ( strcmp( argv[i], "--print-config" ) == 0 ){
			printConfig(config);
			return 0;
		} else if ( strcmp( argv[i], "--help" ) == 0 ){
			printHelp();
			return 0;
		} else {
			printf("Unknown option %s\n", argv[i]);
		}
	}

	if (path_exists( config->userConfig ) ){
		printf("User config file exists but still needs to be parsed\n");
	}

	if (path_exists( config->localConfig ) ){
		printf("Local config file exists but still needs to be parsed\n");
	}

	if ( !db_file_exists(config->database) ){
		printf("Creating DB");
		db_create(config->database);
	}

	listener = proxy_startListener(config->port);
	
	if ( listener < 0 ){
		return 1;
	}

	while ( true ){
		struct sockaddr_in addr;
		socklen_t addrlen = sizeof(addr);
		int client = 0;
		Request *request = NULL;
		Response *response = NULL;
		char *responseStr;
		printf("Listening on port %i\n", config->port);

		if ((client = accept(listener, (struct sockaddr *)&addr, 
				&addrlen))<0) {
			perror("accept");
			return 0;
		}

		//I think eventually I'd like a different thread here for each request
		//Not sure how to do that yet though so I'll keep everything on the main
		//thread

		request = newRequestFromSocket(client);
	
		// If the host is not defined, then it is not a proxy request
		// Note that the host here is where the request should be sent, not
		// necesarily the hosts header
		if ( strcmp( request->host, "" ) == 0 ){
			response = webserverGetResponse(request);
		} else {
			response = upstreamGetResponse(request);
		}

		responseStr = responseToString( response );

		// I'm also not convinced that strlen is the best function to use here
		// When we get to dealing with binary requests / responses, they may
		// well have null characters in them
		send(client , responseStr, strlen(responseStr) , 0 );

		printf( "\n1\n" );
		close(client);
		printf( "\n2\n" );
		freeRequest( request );
		printf( "\n3\n" );
		freeResponse( response );
		printf( "\n4\n" );
		free(responseStr);
		printf( "\n5\n" );

		//If this is an https request - this is the first part
		//if ( strcmp( request->method, "CONNECT" ) == 0 ){

		//	// I am basically doing the same thing that mitmproxy does here
		//	// We start by responding with 200 Connection Established which
		//	// in a normal proxy would mean that we have established a
		//	// connection with the remote host. However, we haven't because we
		//	// are going to pretend to be the host to the client and pretend to
		//	// be the client to the host

		//	response = newResponse();
		//	connectionEstablished(response);
		//	responseStr = responseToString(response);
		//	send(new_socket , responseStr, strlen(responseStr) , 0 );


		//	char line[1024] = {'\0'};
		//	//a length of 2 will indicate an empty line which will split the headers
		//	//from the body (if there is a body)
		//	int valread = read( new_socket, line, 1024);
		//	while (valread > 0){
		//		printf("%s", line);
		//		//I believe at this point all the headers are done.
		//		valread = read( new_socket , line, 1024);
		//	}



		//}

	}

	return 0;
}
